/*
xstradd: concatenate arbitrary number of C strings
Copyright (C) 2000 Ed Cashin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<stdarg.h>
#include	<errno.h>
#include	"xtextadd.h"
#ifdef	LEAK_DETECTION
#include	"leakfind.h"
#endif

#ifndef	PROGNAME
#define	PROGNAME	"test"
#endif
#ifndef	fatal_error
#define	fatal_error(s)	do {					\
    fprintf(stderr, PROGNAME ": Error in %s (%s): %s\n",	\
	    __FUNCTION__, s, strerror(errno));			\
    exit(EXIT_FAILURE);						\
} while (0)
#endif

typedef struct elc_str_listnode_struct {
  struct elc_str_listnode_struct	*next;
  void					*data;
  int					len;
} slnode_t;

inline static void free_slnodes(slnode_t *np, int freedata)
{
    if (! np)
      return;
    free_slnodes(np->next, freedata);
    if (freedata)
      free(np->data);
    free(np);
}

char *xtextadd(const char *str1, ...)
{
    va_list	ap;		/* arg pointer */
    char	*str;
    char	*newstr;
    char	*cp;
    slnode_t	head;
    slnode_t	*curr		 = &head;
    int		totalspace	 = 1;

    head.next	 = NULL;
    head.data	 = (void *) str1;
    head.len	 = strlen(str1);
    totalspace	 += head.len;

    va_start(ap, str1);
    while ( (str = va_arg(ap, char *)) ) {
      curr->next	 = malloc(sizeof(slnode_t));
      if (! curr->next)
	fatal_error("malloc new lnode");
      curr		 = curr->next;
      curr->next	 = NULL;
      curr->data	 = str;
      curr->len		 = strlen(str);
      totalspace         += curr->len;
    }
    va_end(ap);

    newstr	 = malloc(sizeof(char) * totalspace);
    if (! newstr)
      fatal_error("malloc new string");

    cp		 = newstr;
    for (curr = &head; curr; curr = curr->next) {
      strcpy(cp, curr->data);
      cp	 += curr->len;
    }

    free_slnodes(head.next, 0);
    return newstr;
}
